import * as mc from '@minecraft/server';
import { system, world, ItemStack, Entity, BlockPermutation} from "@minecraft/server";

//Necromancy-Infested-Oozing
mc.world.afterEvents.entityDie.subscribe(data => {
    const entity = data?.deadEntity
    if (entity.isValid()) {
        const entityDim = entity?.dimension

    function getRandomInt(max) {
        return Math.floor(Math.random() * max);
    }

    const damagingEntity = data.damageSource.damagingEntity;
    const weapon = damagingEntity.getComponent('equippable')?.getEquipment('Mainhand');
    if(weapon?.typeId === 'minecraft:netherite_sword') {
        weapon.setLore(["That's Overkill"])
        damagingEntity.getComponent('equippable')?.setEquipment('Mainhand', weapon)
    }
        //Necromancy
        if (entity?.hasTag("deathMarked")) {

            if( getRandomInt(4) < 0.5 ) {
                entityDim.spawnEntity('minecraft:creeper',entity.location);
            }
            else {
                entityDim.spawnEntity('minecraft:zombie',entity.location);
            }
        }
        //Infested
        if (entity?.hasTag("infested")) {

            if( getRandomInt(3) > 1 ) {
                entityDim.spawnEntity('minecraft:silverfish',entity.location);
                entityDim.spawnEntity('minecraft:silverfish', entity.location);
            }
        }
        //Oozing
        if (entity?.hasTag("oozing") && entity.typeId !== "minecraft:slime") {

            if( getRandomInt(2) < 1 ) {
                entity.runCommand("summon slime ~ ~ ~ facing @s spawn_warm");
                entity.runCommand("summon slime ~ ~ ~ facing @s spawn_warm");
                }
            
        }
    }
})

//Lightning
system.runInterval(() => {

    const allDimensions = [ 'overworld', 'nether', 'the_end' ];
    for (const dimension of allDimensions) {
        const entities = world.getDimension(dimension).getEntities();
        for (const entity of entities) {
            if (entity.hasTag('lightning1')) {
                function getRandomInt(max) {
                    return Math.floor(Math.random() * max);
                }
                const entityLoc = entity.location
                entityLoc.x += getRandomInt(3) + 1.5
                entityLoc.z += getRandomInt(3) + 1.5

                entity.dimension.spawnEntity("minecraft:lightning_bolt", entityLoc)
            }
        };
    };
},100);


//Shulker Bullets-Iron Golem Potion
mc.world.afterEvents.entityHurt.subscribe(data=> {
    let sourceEntity = data.damageSource.damagingEntity
    let hurtEntity = data.hurtEntity
    if (sourceEntity?.typeId == "si:shulker_bullet") {
        sourceEntity.runCommand("particle minecraft:large_explosion ~ ~ ~")
        hurtEntity.addEffect("levitation", 120, { amplifier: 0, showParticles: true})
        sourceEntity.remove();
    }
    if (sourceEntity?.hasTag('ironGolem')) {
        hurtEntity.applyKnockback(0,0,1,1);
        sourceEntity.runCommand("function spell_ironGolem");
    }
    else{
        return null;
    }

})


//Get Locations
function getNeighboringLocations(location, radius) {
    const neighboringLocations = [];
                
    for (let x = location.x - radius; x <= location.x + radius; x++) {
        for (let y = location.y - radius; y <= location.y + radius; y++) {
            for (let z = location.z - radius; z <= location.z + radius; z++) {
                neighboringLocations.push({x, y, z})
            }
        }
    }
                    
    return neighboringLocations;
}

//Dryad-Repel-Magnet

system.runInterval(() => {
    let players = world.getAllPlayers()
        players.forEach(player => {

            if (player.hasTag("dryad")) {

                let dimension = player.dimension;
                let location = player.location;
                
                const radius = 1;
                const blockLoc = getNeighboringLocations(location, radius);
            
                blockLoc.forEach(location=>{
                    //const transformPerm = BlockPermutation.resolve('minecraft:wheat',{growth:7});
                    const block = dimension.getBlock(location);
                    let growthVal = block.permutation.getState('growth');
                    
                    if (growthVal < 7) {
                        growthVal += 1
                        block.setPermutation(block.permutation.withState('growth', growthVal));
                        dimension.spawnParticle('minecraft:crop_growth_emitter',location)
                    }
                        
                })   
            }   

            if (player.hasTag("alchemist")) {
                let loc1 = player.location;
                let entityK = player.dimension.getEntities({ location: loc1 , maxDistance: 5 });
                entityK.forEach(entity =>{
                    if (entity.typeId == ("minecraft:arrow")) {
                        entity.remove()
                    }
                    else if (entity.hasTag("knockback")) {

                        let loc2 = entity.location;
                        const locX = (loc2.x - loc1.x) * 0.03
                        const locZ = (loc2.z - loc1.z) * 0.03
                        const direction = { x: locX, y: 0, z: locZ };
                        player.runCommand("function alchemistKnockback")
                        entity.applyImpulse(direction);
                                
                    }  
                }) 
            }
            if (player.hasTag("magnetized")) {
                let loc1 = player.location
                let items = player.dimension.getEntities({ location: loc1, maxDistance: 30 })
                items.forEach(item=>{
                    if (item.typeId == "minecraft:item") {
                        let loc2 = item.location
                        let locX = (loc1.x  - loc2.x ) * 0.01
                        let locZ =  (loc1.z  - loc2.z ) * 0.01
                        const directionVal = { x: locX, y: 0, z: locZ}
                        
                        item.applyImpulse(directionVal)
                    }
                })

            }
        })
}, 1)

//Growth Potion
mc.world.afterEvents.projectileHitBlock.subscribe(proj =>{
   
    if (proj.projectile.typeId === 'si:splash_bottle_growth') {
        
        let dimension = proj.dimension
        let location = proj.location;
        
        const radius = 2;
        const blockLoc = getNeighboringLocations(location, radius);
    
        blockLoc.forEach(location=>{
            //const transformPerm = BlockPermutation.resolve('minecraft:wheat',{growth:7});
            const block = dimension.getBlock(location);
            let growthVal = block.permutation.getState('growth');
            if (growthVal < 6) {
                growthVal += 2
                block.setPermutation(block.permutation.withState('growth', growthVal));
                dimension.spawnParticle('minecraft:crop_growth_emitter',location)
            }
            if (growthVal == 6) {
                growthVal += 1
                block.setPermutation(block.permutation.withState('growth', growthVal));
                dimension.spawnParticle('minecraft:crop_growth_emitter',location)
            }
                
        })   
      
    }
    
})

